"""J.A.R.V.I.S. reminder background task handler."""
from asyncio import to_thread
from datetime import datetime, timedelta

from dis_snek.ext.tasks.task import Task
from dis_snek.ext.tasks.triggers import IntervalTrigger

import jarvis
from jarvis.db.models import Reminder
from jarvis.utils import build_embed


async def _remind() -> None:
    """J.A.R.V.I.S. reminder blocking task."""
    reminders = Reminder.objects(remind_at__lte=datetime.utcnow() + timedelta(seconds=30))
    for reminder in reminders:
        if reminder.remind_at <= datetime.utcnow():
            user = await jarvis.jarvis.fetch_user(reminder.user)
            if not user:
                reminder.delete()
                continue
            embed = build_embed(
                title="You have a reminder",
                description=reminder.message,
                fields=[],
            )
            embed.set_author(
                name=user.name + "#" + user.discriminator, icon_url=user.display_avatar.url
            )
            embed.set_thumbnail(url=user.display_avatar.url)
            try:
                await user.send(embed=embed)
            except Exception:
                guild = jarvis.jarvis.fetch_guild(reminder.guild)
                channel = guild.get_channel(reminder.channel) if guild else None
                if channel:
                    await channel.send(f"{user.mention}", embed=embed)
            finally:
                reminder.delete()


@Task.create(trigger=IntervalTrigger(seconds=15))
async def remind() -> None:
    """J.A.R.V.I.S. reminder background task."""
    await to_thread(_remind)
